/*
 * sqlstuff.c - all the deal with postgres db
 * wildcat - 2006
 * dt - 2010
 *
 */


#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "libpq-fe.h"

#include "sqlstuff.h" /* user, s_conf */

static void
exit_nicely(PGconn *conn)
{
    PQfinish(conn);
    exit(1);
}


struct user * ret_user(struct s_conf *conf, const char *l)
{
   	PGconn     	*conn;
        PGresult   	*res;
	int        	rec_count;
	char  		*query;

	char		*login = (char *) l; /* no warning, be happy */
	struct user	*u;

	conn = PQsetdbLogin((const char*)conf->hostname,
                     (const char*)conf->port,
                     NULL,
                     NULL,
                     (const char*)conf->db,
                     (const char*)conf->login,
                     (const char*)conf->passw);

	if (PQstatus(conn) != CONNECTION_OK)
    	{
        	fprintf(stderr, "Connection to database failed: %s",
                	PQerrorMessage(conn));
        	exit_nicely(conn);
    	}

	/* forge query */

	if (strlen(l) > 50) login[50] = 0; /* Limit of sql field (very bad tricks of *login, uuuugly) */

	query = (char*) malloc(sizeof(char) * strlen(SQL_GETUSER) + strlen(l) + 1);
	sprintf(query, SQL_GETUSER, l);

	res = PQexec(conn, query);

	if (PQresultStatus(res) != PGRES_TUPLES_OK)
     	{
         fprintf(stderr, "osfw-auth-pgsql: SELECT failed: %s", PQerrorMessage(conn));
         PQclear(res);
         exit_nicely(conn);
	 return NULL;
     	}

	rec_count = PQntuples(res);

	if (rec_count == 0 || rec_count > 1)
	{
         	fprintf(stderr, "osfw-auth-pgsql: User not found or query invalid (%d)", rec_count);

	/*       Das stellt fuer uns keinen Fehlerfall dar. Wenn es den User nicht gibt soll es 
	 	zwar loggen, aber nicht brechen. Daher folgender bereich auskommentiert.  
	
	 	PQclear(res);
         	exit_nicely(conn);
         	return NULL; */
		
		// Wir geben NULL zurück, dann bricht es in auth-pgsql.c korrekt ab

		u = (struct user *) malloc(sizeof(struct user));
		u->id = NULL;
		u->login = NULL;
		u->passw = NULL;
		return u;		

	} else {

		u = (struct user *) malloc(sizeof(struct user));

		u->id = atoi(PQgetvalue(res, 0, 0)); 

		u->login = (char *) malloc(sizeof(char) * strlen(PQgetvalue(res, 0, 1)) + 1);
		u->passw = (char *) malloc(sizeof(char) * strlen(PQgetvalue(res, 0, 2)) + 1);

		strncpy(u->login, PQgetvalue(res, 0, 1), strlen(PQgetvalue(res, 0, 1)) + 1);
		strncpy(u->passw, PQgetvalue(res, 0, 2), strlen(PQgetvalue(res, 0, 2)) + 1);

		PQclear(res);
		PQfinish(conn);

		return u;
	}
}
